<?php
namespace Ak\Mailchimp\Gateway;

use Ak\Mailchimp\Entity\MailingList;
use Ak\Mailchimp\Response\GetListMembersResponse;
use Ak\Mailchimp\Response\GetListsResponse;

/**
 * Pruža pristup mailing listama.
 *
 * @package Ak\Mailchimp\Gateway
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://bitbucket.org/autokreso/mailchimp
 */
final class MailingListGateway extends AbstractGateway
{
    /**
     * Vraća sve mailing liste.
     *
     * @return MailingList[]
     */
    public function getAll(): array
    {
        $data = $this->request('GET', 'lists');
        /** @var GetListsResponse $response */
        $response = $this->hydrator->hydrate($data, new GetListsResponse());

        return $response->getLists();
    }

    /**
     * Vraća mailing listu po id-u.
     *
     * @param string $list_id
     *
     * @return MailingList
     */
    public function getList(string $list_id): MailingList
    {
        if ($data = $this->request('GET', 'lists/' . $list_id)) {
            if ($data['status'] >= 200 && $data['status'] < 300) {
                /** @var MailingList $list */
                $list = $this->hydrator->hydrate($data, new MailingList());
            }
        }

        return $list ?? null;
    }

    /**
     * Vraća članove date liste.
     *
     * @param MailingList $list
     *
     * @return MailingList\Member[]
     */
    public function getListMembers(MailingList $list): array
    {
        $data = $this->request('GET', 'lists/' . $list->getId() . '/members');
        /** @var GetListMembersResponse $response */
        $response = $this->hydrator->hydrate($data, new GetListMembersResponse());

        return $response->getMembers();
    }

    /**
     * Vraća člana date liste s datom e-mail adresom.
     *
     * @param MailingList $list
     * @param string $email
     *
     * @return MailingList\Member|null
     */
    public function getListMember(MailingList $list, string $email): ?MailingList\Member
    {
        if ($data = $this->request('GET', 'lists/' . $list->getId() . '/members/' . md5($email))) {
            /** @var MailingList\Member $member */
            $member = $this->hydrator->hydrate($data, new MailingList\Member());
        }

        return $member ?? null;
    }

    /**
     * Dodaje novu e-mail adresu u datu mailing listu.
     *
     * @param MailingList $list
     * @param string $email
     *
     * @return MailingList\Member|null
     */
    public function addListMember(MailingList $list, string $email): ?MailingList\Member
    {
        $data = $this->request('PUT', 'lists/' . $list->getId() . '/members/' . md5($email), [
            'email_address' => $email,
            'status' => 'subscribed',
        ]);

        if ($data) {
            /** @var MailingList\Member $member */
            $member = $this->hydrator->hydrate($data, new MailingList\Member());
        }

        return $member ?? null;
    }
}
