<?php
namespace AkTest\Mailchimp\Gateway;

use Ak\Mailchimp\Entity\MailingList;
use Ak\Mailchimp\Gateway\MailingListGateway;
use GuzzleHttp\Psr7\Response;
use PHPUnit\Framework\MockObject\MockObject;

final class MailingListGatewayTest extends AbstractGatewayTestCase
{
    public function testCanGetAllLists()
    {
        $container = $this->getContainerWithMockedHttpClient([
            __DIR__ . '/../data/get-lists.dat',
        ]);

        /** @var MailingListGateway $gateway */
        $gateway = $container->get('mailchimp.lists');
        $lists = $gateway->getAll();

        /** @noinspection PhpParamsInspection */
        $this->assertCount(2, $lists);
        /** @noinspection PhpParamsInspection */
        $this->assertInstanceOf(MailingList::class, $lists[0]);
        /** @noinspection PhpParamsInspection */
        $this->assertInstanceOf(MailingList::class, $lists[1]);
        $this->assertEquals('12f21002ad', $lists[0]->getId());
        $this->assertEquals('312f28f047', $lists[1]->getId());
    }

    public function testCanGetListById()
    {
        $container = $this->getContainerWithMockedHttpClient([
            __DIR__ . '/../data/get-lists-a73560215d.dat',
        ]);

        /** @var MailingListGateway $gateway */
        $gateway = $container->get('mailchimp.lists');
        $list = $gateway->getList('a73560215d');

        $this->assertEquals('a73560215d', $list->getId());
        $this->assertEquals('noreply@autokreso.hr', $list->getCampaignDefaults()->getFromEmail());
    }

    public function testCanGetListMembers()
    {
        $container = $this->getContainerWithMockedHttpClient([
            __DIR__ . '/../data/get-lists-a73560215d-members.dat',
        ]);

        /** @var MailingList|MockObject $list */
        $list = $this->createMock(MailingList::class);
        $list->expects($this->once())
            ->method('getId')
            ->willReturn('a73560215d');

        /** @var MailingListGateway $gateway */
        $gateway = $container->get('mailchimp.lists');
        $members = $gateway->getListMembers($list);

        $this->assertCount(1, $members);
        $this->assertEquals('danijel@ep.hr', $members[0]->getEmailAddress());
    }

    public function testCanGetSingleListMember()
    {
        $container = $this->getContainerWithMockedHttpClient([
            __DIR__ . '/../data/get-lists-a73560215d-members-4ce8a82885d2c7f9ff45533531efbd95.dat',
        ]);

        /** @var MailingList|MockObject $list */
        $list = $this->createMock(MailingList::class);
        $list->expects($this->once())
            ->method('getId')
            ->willReturn('a73560215d');

        /** @var MailingListGateway $gateway */
        $gateway = $container->get('mailchimp.lists');
        $member = $gateway->getListMember($list, 'danijel@ep.hr');

        $this->assertInstanceOf(MailingList\Member::class, $member);
        $this->assertEquals(md5('danijel@ep.hr'), $member->getId());
        $this->assertEquals('danijel@ep.hr', $member->getEmailAddress());
    }

    public function testWillReturnNullForInvalidListMember()
    {
        $container = $this->getContainerWithMockedHttpClient([
            new Response(404),
        ]);

        /** @var MailingList|MockObject $list */
        $list = $this->createMock(MailingList::class);
        $list->expects($this->once())
            ->method('getId')
            ->willReturn('a73560215d');

        /** @var MailingListGateway $gateway */
        $gateway = $container->get('mailchimp.lists');
        $member = $gateway->getListMember($list, 'invalid');

        $this->assertNull($member);
    }

    public function testCanAddMembersToList()
    {
        $container = $this->getContainerWithMockedHttpClient([
            __DIR__ . '/../data/post-lists-a73560215d-members.dat',
        ]);

        /** @var MailingList|MockObject $list */
        $list = $this->createMock(MailingList::class);
        $list->expects($this->once())
            ->method('getId')
            ->willReturn('a73560215d');

        /** @var MailingListGateway $gateway */
        $gateway = $container->get('mailchimp.lists');
        $member = $gateway->addListMember($list, 'hipnaba@gmail.com');

        $this->assertInstanceOf(MailingList\Member::class, $member);
        $this->assertEquals('hipnaba@gmail.com', $member->getEmailAddress());
        $this->assertEquals('subscribed', $member->getStatus());
    }
}